package cz.jzitnik.gameoflife.controllers;

import cz.jzitnik.gameoflife.game.GameOfLife;
import javafx.animation.KeyFrame;
import javafx.animation.Timeline;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.scene.control.*;
import javafx.scene.layout.GridPane;
import javafx.scene.paint.Color;
import javafx.scene.shape.Rectangle;
import javafx.util.Duration;

import java.text.DecimalFormat;

public class GameOfLifeController {
    private static final int CELL_SIZE = 25;
    private GameOfLife game;
    private Rectangle[][] cells;
    private int height = 25;
    private int width = 25;

    @FXML
    private GridPane gridPane;
    @FXML
    private Button startButton;
    @FXML
    private Label timeLabel;
    @FXML
    private TextField rateTextField;
    @FXML
    private TextField widthField;
    @FXML
    private TextField heightField;
    @FXML
    private Button resizeButton;

    private Timeline timeline;

    @FXML
    public void initialize() {
        setupGame();
        setupTimeline();
        startButton.setOnAction(_ -> toggleGame());
        resizeButton.setOnAction(_ -> resizeBoard());

        rateTextField.textProperty().addListener((_, _, newValue) -> {
            try {
                long newRate = Long.parseLong(newValue);
                if (newRate > 0) {
                    updateTimelineRate(newRate);
                }
            } catch (NumberFormatException ignored) {
            }
        });
    }

    private void setupGame() {
        game = new GameOfLife(height, width);
        cells = new Rectangle[height][width];
        setupGrid();
    }

    private void setupGrid() {
        gridPane.getChildren().clear();
        for (int i = 0; i < height; i++) {
            for (int j = 0; j < width; j++) {
                Rectangle rect = new Rectangle(CELL_SIZE, CELL_SIZE);
                rect.setFill(Color.LIGHTGRAY);
                rect.setStroke(Color.BLACK);
                final int row = i, col = j;
                rect.setOnMouseClicked(_ -> toggleCell(row, col));
                cells[i][j] = rect;
                gridPane.add(rect, j, i);
            }
        }
    }

    private void toggleCell(int row, int col) {
        game.toggleCell(row, col);
        updateGrid();
    }

    private void updateGrid() {
        for (int i = 0; i < height; i++) {
            for (int j = 0; j < width; j++) {
                cells[i][j].setFill(game.getCell(i, j) ? Color.BLACK : Color.LIGHTGRAY);
            }
        }
    }

    private void toggleGame() {
        if (timeline.getStatus() == Timeline.Status.RUNNING) {
            stopGame();
        } else {
            startGame();
        }
    }

    private void startGame() {
        timeline.play();
        startButton.setText("Stop");
        rateTextField.setDisable(true);
        widthField.setDisable(true);
        heightField.setDisable(true);
    }

    private void stopGame() {
        timeline.stop();
        startButton.setText("Start");
        rateTextField.setDisable(false);
        widthField.setDisable(false);
        heightField.setDisable(false);
    }

    private void setupTimeline() {
        long updateRate = 500;
        timeline = new Timeline(new KeyFrame(Duration.millis(updateRate), this::update));
        timeline.setCycleCount(Timeline.INDEFINITE);
    }

    private void updateTimelineRate(long rate) {
        timeline.getKeyFrames().clear();
        timeline.getKeyFrames().add(new KeyFrame(Duration.millis(rate), this::update));
    }

    private void update(ActionEvent actionEvent) {
        if (game.isBoardEmpty()) {
            stopGame();
            return;
        }
        long startTime = System.nanoTime();
        game.nextGeneration();
        updateGrid();
        double timeTookInMs = (double) (System.nanoTime() - startTime) / 1_000_000;
        DecimalFormat df = new DecimalFormat("0.00");
        timeLabel.setText("Took " + df.format(timeTookInMs) + "ms to update.");
    }

    private void resizeBoard() {
        try {
            int newWidth = Integer.parseInt(widthField.getText());
            int newHeight = Integer.parseInt(heightField.getText());
            if (newWidth > 0 && newHeight > 0) {
                width = newWidth;
                height = newHeight;
                setupGame();
            }
        } catch (NumberFormatException ignored) {
        }
    }
}
